#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-
import ast
import base64
import hashlib
import json
import logging
import urllib3
import time
from io import BytesIO
from PIL import Image

FORMAT = '%(asctime)s-%(levelname)s: %(message)s'
logging.basicConfig(level=logging.INFO, format=FORMAT)
logger = logging.getLogger(__name__)


class WebcamInterface():

    def __init__(self, camera_id_a, identification_id_a):
        self._user = 'user'            # set
        self._password = 'password'    # set
        self._host = 'xxx.xxx.xxx.xxx' # set
        self._port = 'xxxx'            # set
        self._camera_id_a = camera_id_a
        self._identification_id_a = identification_id_a
        self._http_pool = urllib3.PoolManager()
        self._HTTP_RESPONSE_OK = 200
        self._HTTP_RESPONSE_WAIT_SEC = 1
        self._RETRY_COUNT_MAX = 3

    def get_camera_capture(self, camera_id):
        """
        Camera capture acquisition process

        Parameters
        --------------------
        ID of the camera for which you want to take a camera capture

        Returns
        --------------------
        camera capture data
        """
        img_data = None
        if camera_id in self._camera_id_a:
            index = self._camera_id_a.index(camera_id)
            identification_id = self._identification_id_a[index]
            for retry_count in range(self._RETRY_COUNT_MAX):
                auth_code = self.create_auth_code()
                if auth_code is None:
                    continue
                cookie_login_url = 'http://{}:{}/api/cookieLogin'.format(self._host, self._port)
                body_data = {"auth": auth_code.decode()}
                headers = {'Content-Type': 'application/json'}

                self._http_pool.request('POST', cookie_login_url,
                                        body=json.dumps(body_data).encode(),
                                        headers=headers)
                camera_apt_url = ('http://{}:{}/ec2/cameraThumbnail?auth={}'
                                  '&cameraId={}&time=now&rotate=0&height=-1'
                                  '&width=-1&imageFormat=jpg&method=precise')
                camera_apt_resp = self._http_pool.request('GET', camera_apt_url.format(
                                                          self._host, self._port, auth_code.decode(), identification_id))
                if camera_apt_resp.status == self._HTTP_RESPONSE_OK:
                    img_data = Image.open(BytesIO(camera_apt_resp.data))
                    self._http_pool.clear()
                    break
                else:
                    time.sleep(self._HTTP_RESPONSE_WAIT_SEC)
            if img_data is None:
                logger.error("Unable to get camera %d capture", camera_id)
            # else:
            #    img_data = img_data.resize((640, 480))
        return img_data

    def create_auth_code(self):
        """
        Authentication code creation process

        Returns
        --------------------
        authentication code
        """
        auth_digest = None
        try:
            resp = self._http_pool.request('GET', 'http://{}:{}/api/getNonce'.format(self._host, self._port))
            dic = ast.literal_eval(resp.data.decode())
            reply = dic["reply"]
            nonce = reply["nonce"]
            realm = reply["realm"]
            digest_str = "{}:{}:{}".format(self._user, realm, self._password)
            digest = hashlib.md5(digest_str.encode()).hexdigest()
            partial_ha2 = hashlib.md5(b'GET:').hexdigest()
            simplified_ha2_str = "{}:{}:{}".format(digest, nonce, partial_ha2)
            simplified_ha2 = hashlib.md5(simplified_ha2_str.encode()).hexdigest()
            auth_digest_str = "{}:{}:{}".format(self._user, nonce, simplified_ha2)
            auth_digest = base64.b64encode(auth_digest_str.encode())
        except self._http_pool.exceptions.HTTPError as e:
            logger.error(e)

        return auth_digest

        



